use Evo::Base -strict;
use Test::More;
use Railway::Builder;

ok my $bldr = Railway::Builder->new;

isa_ok $bldr->build_way,   $bldr->way_class,   'way';
isa_ok $bldr->build_train, $bldr->train_class, 'train';
isa_ok $bldr->build_train->way, $bldr->way_class, 'train is build with a way';

done_testing;

1;
