use Evo::Base -base;
use Test::More tests => 2;
use Evo::Wrappers ':all';
use Evo::Manager;

my $mngr = Evo::Manager->new;

my ($current, @input, $curry);

# build_handler
$curry = Evo::Manager->singleton->curry_wrappers(w_dsl_init);
my $train = $mngr->build_train()->power(1);
$curry->(
  sub {
    is $mngr->current('train'), $train, "right current train";
    is shift, 'foo', '2 arg';
  }
)->($train, 'foo');


done_testing;
