use Evo::Base -strict;
use Test::More;
use Evo::Manager;

my $m = Evo::Manager->singleton;

my $way = $m->build_way;
$way->add(sub { });
is ref $way->get(0), 'CODE', 'CODE';
is $way->get(0), undef, 'cleared';

done_testing;
